#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest

# For no Pydantic environment, we need to skip the tests
pytest.importorskip("google.cloud.aiplatform_v1")

from google.api_core.gapic_v1.method import DEFAULT

from airflow.providers.google.cloud.hooks.vertex_ai.endpoint_service import EndpointServiceHook

from unit.google.cloud.utils.base_gcp_mock import (
    mock_base_gcp_hook_default_project_id,
    mock_base_gcp_hook_no_default_project_id,
)

TEST_GCP_CONN_ID: str = "test-gcp-conn-id"
TEST_REGION: str = "test-region"
TEST_PROJECT_ID: str = "test-project-id"
TEST_ENDPOINT: dict = {}
TEST_ENDPOINT_ID: str = "1234567890"
TEST_ENDPOINT_NAME: str = "test_endpoint_name"
TEST_DEPLOYED_MODEL: dict = {}
TEST_DEPLOYED_MODEL_ID: str = "test-deployed-model-id"
TEST_TRAFFIC_SPLIT: dict = {}
TEST_UPDATE_MASK: dict = {}

BASE_STRING = "airflow.providers.google.common.hooks.base_google.{}"
ENDPOINT_SERVICE_STRING = "airflow.providers.google.cloud.hooks.vertex_ai.endpoint_service.{}"


class TestEndpointServiceWithDefaultProjectIdHook:
    def setup_method(self):
        with mock.patch(
            BASE_STRING.format("GoogleBaseHook.__init__"), new=mock_base_gcp_hook_default_project_id
        ):
            self.hook = EndpointServiceHook(gcp_conn_id=TEST_GCP_CONN_ID)

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_create_endpoint(self, mock_client) -> None:
        self.hook.create_endpoint(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint=TEST_ENDPOINT,
            endpoint_id=TEST_ENDPOINT_ID,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.create_endpoint.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                endpoint=TEST_ENDPOINT,
                endpoint_id=TEST_ENDPOINT_ID,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_REGION,
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_delete_endpoint(self, mock_client) -> None:
        self.hook.delete_endpoint(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint=TEST_ENDPOINT_NAME,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.delete_endpoint.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.endpoint_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.endpoint_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_REGION,
            TEST_ENDPOINT_NAME,
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_deploy_model(self, mock_client) -> None:
        self.hook.deploy_model(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint=TEST_ENDPOINT_NAME,
            deployed_model=TEST_DEPLOYED_MODEL,
            traffic_split=TEST_TRAFFIC_SPLIT,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.deploy_model.assert_called_once_with(
            request=dict(
                endpoint=mock_client.return_value.endpoint_path.return_value,
                deployed_model=TEST_DEPLOYED_MODEL,
                traffic_split=TEST_TRAFFIC_SPLIT,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.endpoint_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_REGION,
            TEST_ENDPOINT_NAME,
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_get_endpoint(self, mock_client) -> None:
        self.hook.get_endpoint(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint=TEST_ENDPOINT_NAME,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.get_endpoint.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.endpoint_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.endpoint_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_REGION,
            TEST_ENDPOINT_NAME,
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_list_endpoints(self, mock_client) -> None:
        self.hook.list_endpoints(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.list_endpoints.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                filter=None,
                page_size=None,
                page_token=None,
                read_mask=None,
                order_by=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_REGION,
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_undeploy_model(self, mock_client) -> None:
        self.hook.undeploy_model(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint=TEST_ENDPOINT_NAME,
            deployed_model_id=TEST_DEPLOYED_MODEL_ID,
            traffic_split=TEST_TRAFFIC_SPLIT,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.undeploy_model.assert_called_once_with(
            request=dict(
                endpoint=mock_client.return_value.endpoint_path.return_value,
                deployed_model_id=TEST_DEPLOYED_MODEL_ID,
                traffic_split=TEST_TRAFFIC_SPLIT,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.endpoint_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_ENDPOINT_NAME
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_update_endpoint(self, mock_client) -> None:
        self.hook.update_endpoint(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint_id=TEST_ENDPOINT_NAME,
            endpoint=TEST_ENDPOINT,
            update_mask=TEST_UPDATE_MASK,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.update_endpoint.assert_called_once_with(
            request=dict(
                endpoint=TEST_ENDPOINT,
                update_mask=TEST_UPDATE_MASK,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )


class TestEndpointServiceWithoutDefaultProjectIdHook:
    def setup_method(self):
        with mock.patch(
            BASE_STRING.format("GoogleBaseHook.__init__"), new=mock_base_gcp_hook_no_default_project_id
        ):
            self.hook = EndpointServiceHook(gcp_conn_id=TEST_GCP_CONN_ID)

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_create_endpoint(self, mock_client) -> None:
        self.hook.create_endpoint(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint=TEST_ENDPOINT,
            endpoint_id=TEST_ENDPOINT_ID,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.create_endpoint.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                endpoint=TEST_ENDPOINT,
                endpoint_id=TEST_ENDPOINT_ID,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_REGION,
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_delete_endpoint(self, mock_client) -> None:
        self.hook.delete_endpoint(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint=TEST_ENDPOINT_NAME,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.delete_endpoint.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.endpoint_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.endpoint_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_REGION,
            TEST_ENDPOINT_NAME,
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_deploy_model(self, mock_client) -> None:
        self.hook.deploy_model(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint=TEST_ENDPOINT_NAME,
            deployed_model=TEST_DEPLOYED_MODEL,
            traffic_split=TEST_TRAFFIC_SPLIT,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.deploy_model.assert_called_once_with(
            request=dict(
                endpoint=mock_client.return_value.endpoint_path.return_value,
                deployed_model=TEST_DEPLOYED_MODEL,
                traffic_split=TEST_TRAFFIC_SPLIT,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.endpoint_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_REGION,
            TEST_ENDPOINT_NAME,
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_get_endpoint(self, mock_client) -> None:
        self.hook.get_endpoint(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint=TEST_ENDPOINT_NAME,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.get_endpoint.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.endpoint_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.endpoint_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_REGION,
            TEST_ENDPOINT_NAME,
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_list_endpoints(self, mock_client) -> None:
        self.hook.list_endpoints(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.list_endpoints.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                filter=None,
                page_size=None,
                page_token=None,
                read_mask=None,
                order_by=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_REGION,
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_undeploy_model(self, mock_client) -> None:
        self.hook.undeploy_model(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint=TEST_ENDPOINT_NAME,
            deployed_model_id=TEST_DEPLOYED_MODEL_ID,
            traffic_split=TEST_TRAFFIC_SPLIT,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.undeploy_model.assert_called_once_with(
            request=dict(
                endpoint=mock_client.return_value.endpoint_path.return_value,
                deployed_model_id=TEST_DEPLOYED_MODEL_ID,
                traffic_split=TEST_TRAFFIC_SPLIT,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.endpoint_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_ENDPOINT_NAME
        )

    @mock.patch(ENDPOINT_SERVICE_STRING.format("EndpointServiceHook.get_endpoint_service_client"))
    def test_update_endpoint(self, mock_client) -> None:
        self.hook.update_endpoint(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            endpoint_id=TEST_ENDPOINT_NAME,
            endpoint=TEST_ENDPOINT,
            update_mask=TEST_UPDATE_MASK,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.update_endpoint.assert_called_once_with(
            request=dict(
                endpoint=TEST_ENDPOINT,
                update_mask=TEST_UPDATE_MASK,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest

from airflow.providers.common.compat.sdk import AirflowException, TaskDeferred
from airflow.providers.google.cloud.hooks.dataflow import DataflowJobStatus
from airflow.providers.google.cloud.sensors.dataflow import (
    DataflowJobAutoScalingEventsSensor,
    DataflowJobMessagesSensor,
    DataflowJobMetricsSensor,
    DataflowJobStatusSensor,
)
from airflow.providers.google.cloud.triggers.dataflow import (
    DataflowJobAutoScalingEventTrigger,
    DataflowJobMessagesTrigger,
    DataflowJobMetricsTrigger,
    DataflowJobStatusTrigger,
)

TEST_TASK_ID = "task_id"
TEST_JOB_ID = "test_job_id"
TEST_PROJECT_ID = "test_project"
TEST_LOCATION = "us-central1"
TEST_GCP_CONN_ID = "test_gcp_conn_id"
TEST_IMPERSONATION_CHAIN = ["ACCOUNT_1", "ACCOUNT_2", "ACCOUNT_3"]


class TestDataflowJobStatusSensor:
    @pytest.mark.parametrize(
        ("expected_status", "current_status", "sensor_return"),
        [
            (DataflowJobStatus.JOB_STATE_DONE, DataflowJobStatus.JOB_STATE_DONE, True),
            (DataflowJobStatus.JOB_STATE_DONE, DataflowJobStatus.JOB_STATE_RUNNING, False),
        ],
    )
    @mock.patch("airflow.providers.google.cloud.sensors.dataflow.DataflowHook")
    def test_poke(self, mock_hook, expected_status, current_status, sensor_return):
        mock_get_job = mock_hook.return_value.get_job
        task = DataflowJobStatusSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            expected_statuses=expected_status,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_get_job.return_value = {"id": TEST_JOB_ID, "currentState": current_status}
        results = task.poke(mock.MagicMock())

        assert sensor_return == results

        mock_hook.assert_called_once_with(
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_get_job.assert_called_once_with(
            job_id=TEST_JOB_ID, project_id=TEST_PROJECT_ID, location=TEST_LOCATION
        )

    @mock.patch("airflow.providers.google.cloud.sensors.dataflow.DataflowHook")
    def test_poke_raise_exception(self, mock_hook):
        mock_get_job = mock_hook.return_value.get_job
        task = DataflowJobStatusSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            expected_statuses=DataflowJobStatus.JOB_STATE_RUNNING,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_get_job.return_value = {"id": TEST_JOB_ID, "currentState": DataflowJobStatus.JOB_STATE_CANCELLED}

        with pytest.raises(
            AirflowException,
            match=f"Job with id '{TEST_JOB_ID}' is already in terminal state: "
            f"{DataflowJobStatus.JOB_STATE_CANCELLED}",
        ):
            task.poke(mock.MagicMock())

        mock_hook.assert_called_once_with(
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_get_job.assert_called_once_with(
            job_id=TEST_JOB_ID, project_id=TEST_PROJECT_ID, location=TEST_LOCATION
        )

    @mock.patch("airflow.providers.google.cloud.sensors.dataflow.DataflowJobStatusSensor.poke")
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook")
    def test_execute_enters_deferred_state(self, mock_hook, mock_poke):
        """
        Tests that DataflowJobStatusTrigger will be fired when the DataflowJobStatusSensor
        is executed and deferrable is set to True.
        """
        task = DataflowJobStatusSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            expected_statuses=DataflowJobStatus.JOB_STATE_DONE,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
        )
        mock_hook.return_value.exists.return_value = False
        mock_poke.return_value = False
        with pytest.raises(TaskDeferred) as exc:
            task.execute(None)
        assert isinstance(exc.value.trigger, DataflowJobStatusTrigger), (
            "Trigger is not a DataflowJobStatusTrigger"
        )

    def test_execute_complete_success(self):
        """Tests that the trigger event contains expected values if no callback function is provided."""
        expected_result = True
        task = DataflowJobStatusSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            expected_statuses=DataflowJobStatus.JOB_STATE_DONE,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
        )
        actual_message = task.execute_complete(
            context=None,
            event={
                "status": "success",
                "message": f"Job with id '{TEST_JOB_ID}' has reached an expected state: {DataflowJobStatus.JOB_STATE_DONE}",
            },
        )
        assert actual_message == expected_result

    def test_execute_complete_not_success_status_raises_exception(self):
        """Tests that AirflowException or AirflowSkipException is raised if the trigger event contains an error."""
        task = DataflowJobStatusSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            location=TEST_LOCATION,
            expected_statuses=DataflowJobStatus.JOB_STATE_DONE,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
        )
        with pytest.raises(AirflowException):
            task.execute_complete(context=None, event={"status": "error", "message": "test error message"})


class TestDataflowJobMetricsSensor:
    @pytest.mark.parametrize(
        ("job_current_state", "fail_on_terminal_state"),
        [
            (DataflowJobStatus.JOB_STATE_RUNNING, True),
            (DataflowJobStatus.JOB_STATE_RUNNING, False),
            (DataflowJobStatus.JOB_STATE_DONE, False),
        ],
    )
    @mock.patch("airflow.providers.google.cloud.sensors.dataflow.DataflowHook")
    def test_poke(self, mock_hook, job_current_state, fail_on_terminal_state):
        mock_get_job = mock_hook.return_value.get_job
        mock_fetch_job_metrics_by_id = mock_hook.return_value.fetch_job_metrics_by_id
        callback = mock.MagicMock()

        task = DataflowJobMetricsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=callback,
            fail_on_terminal_state=fail_on_terminal_state,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_get_job.return_value = {"id": TEST_JOB_ID, "currentState": job_current_state}
        results = task.poke(mock.MagicMock())

        assert callback.return_value == results

        mock_hook.assert_called_once_with(
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_fetch_job_metrics_by_id.assert_called_once_with(
            job_id=TEST_JOB_ID, project_id=TEST_PROJECT_ID, location=TEST_LOCATION
        )
        mock_fetch_job_metrics_by_id.return_value.__getitem__.assert_called_once_with("metrics")
        callback.assert_called_once_with(mock_fetch_job_metrics_by_id.return_value.__getitem__.return_value)

    @mock.patch("airflow.providers.google.cloud.sensors.dataflow.DataflowHook")
    def test_poke_raise_exception(self, mock_hook):
        mock_get_job = mock_hook.return_value.get_job
        mock_fetch_job_messages_by_id = mock_hook.return_value.fetch_job_messages_by_id
        callback = mock.MagicMock()

        task = DataflowJobMetricsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=callback,
            fail_on_terminal_state=True,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_get_job.return_value = {"id": TEST_JOB_ID, "currentState": DataflowJobStatus.JOB_STATE_DONE}

        with pytest.raises(
            AirflowException,
            match=f"Job with id '{TEST_JOB_ID}' is already in terminal state: "
            f"{DataflowJobStatus.JOB_STATE_DONE}",
        ):
            task.poke(mock.MagicMock())

        mock_hook.assert_called_once_with(
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_fetch_job_messages_by_id.assert_not_called()
        callback.assert_not_called()

    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook")
    def test_execute_enters_deferred_state(self, mock_hook):
        """
        Tests that DataflowJobMetricsTrigger will be fired when the DataflowJobMetricsSensor
        is executed and deferrable is set to True.
        """
        task = DataflowJobMetricsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
            callback=None,
        )
        mock_hook.return_value.exists.return_value = False
        with pytest.raises(TaskDeferred) as exc:
            task.execute(None)
        assert isinstance(exc.value.trigger, DataflowJobMetricsTrigger), (
            "Trigger is not a DataflowJobMetricsTrigger"
        )

    def test_execute_complete_success_without_callback_function(self):
        """Tests that the trigger event contains expected values if no callback function is provided."""
        expected_result = []
        task = DataflowJobMetricsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
            callback=None,
        )
        actual_message = task.execute_complete(
            context=None,
            event={
                "status": "success",
                "message": f"Detected 2 metrics for job '{TEST_JOB_ID}'",
                "result": [],
            },
        )
        assert actual_message == expected_result

    def test_execute_complete_success_with_callback_function(self):
        """Tests that the trigger event contains expected values if the callback function is provided."""
        expected_result = [
            {
                "name": {"origin": "", "name": "", "context": {}},
                "scalar": 0.0,
                "update_time": "2024-03-20T12:36:05.229Z",
                "kind": "",
                "cumulative": False,
            },
            {
                "name": {"origin": "", "name": "", "context": {}},
                "scalar": 0.0,
                "update_time": "2024-03-20T12:36:05.229Z",
                "kind": "",
                "cumulative": False,
            },
        ]
        task = DataflowJobMetricsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=lambda res: res,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
        )
        actual_result = task.execute_complete(
            context=None,
            event={
                "status": "success",
                "message": f"Detected 2 job messages for job '{TEST_JOB_ID}'",
                "result": expected_result,
            },
        )
        assert actual_result == expected_result

    def test_execute_complete_not_success_status_raises_exception(self):
        """Tests that AirflowException or AirflowSkipException is raised if the trigger event contains an error."""
        task = DataflowJobMetricsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=None,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
        )
        with pytest.raises(AirflowException):
            task.execute_complete(
                context=None, event={"status": "error", "message": "test error message", "result": None}
            )


class TestDataflowJobMessagesSensor:
    @pytest.mark.parametrize(
        ("job_current_state", "fail_on_terminal_state"),
        [
            (DataflowJobStatus.JOB_STATE_RUNNING, True),
            (DataflowJobStatus.JOB_STATE_RUNNING, False),
            (DataflowJobStatus.JOB_STATE_DONE, False),
        ],
    )
    @mock.patch("airflow.providers.google.cloud.sensors.dataflow.DataflowHook")
    def test_poke(self, mock_hook, job_current_state, fail_on_terminal_state):
        mock_get_job = mock_hook.return_value.get_job
        mock_fetch_job_messages_by_id = mock_hook.return_value.fetch_job_messages_by_id
        callback = mock.MagicMock()

        task = DataflowJobMessagesSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=callback,
            fail_on_terminal_state=fail_on_terminal_state,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_get_job.return_value = {"id": TEST_JOB_ID, "currentState": job_current_state}

        results = task.poke(mock.MagicMock())

        assert callback.return_value == results.xcom_value

        mock_hook.assert_called_once_with(
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_fetch_job_messages_by_id.assert_called_once_with(
            job_id=TEST_JOB_ID, project_id=TEST_PROJECT_ID, location=TEST_LOCATION
        )
        callback.assert_called_once_with(mock_fetch_job_messages_by_id.return_value)

    @mock.patch("airflow.providers.google.cloud.sensors.dataflow.DataflowHook")
    def test_poke_raise_exception(self, mock_hook):
        mock_get_job = mock_hook.return_value.get_job
        mock_fetch_job_messages_by_id = mock_hook.return_value.fetch_job_messages_by_id
        callback = mock.MagicMock()

        task = DataflowJobMessagesSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=callback,
            fail_on_terminal_state=True,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_get_job.return_value = {"id": TEST_JOB_ID, "currentState": DataflowJobStatus.JOB_STATE_DONE}

        with pytest.raises(
            AirflowException,
            match=f"Job with id '{TEST_JOB_ID}' is already in terminal state: "
            f"{DataflowJobStatus.JOB_STATE_DONE}",
        ):
            task.poke(mock.MagicMock())

        mock_hook.assert_called_once_with(
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_fetch_job_messages_by_id.assert_not_called()
        callback.assert_not_called()

    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook")
    def test_execute_enters_deferred_state(self, mock_hook):
        """
        Tests that DataflowJobMessagesTrigger will be fired when the DataflowJobMessagesSensor
        is executed and deferrable is set to True.
        """
        task = DataflowJobMessagesSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
            callback=None,
        )
        mock_hook.return_value.exists.return_value = False
        with pytest.raises(TaskDeferred) as exc:
            task.execute(None)
        assert isinstance(exc.value.trigger, DataflowJobMessagesTrigger), (
            "Trigger is not a DataflowJobMessagesTrigger"
        )

    def test_execute_complete_success_without_callback_function(self):
        """Tests that the trigger event contains expected values if no callback function is provided."""
        expected_result = []
        task = DataflowJobMessagesSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
            callback=None,
        )
        actual_message = task.execute_complete(
            context=None,
            event={
                "status": "success",
                "message": f"Detected 2 job messages for job '{TEST_JOB_ID}'",
                "result": [],
            },
        )
        assert actual_message == expected_result

    def test_execute_complete_success_with_callback_function(self):
        """Tests that the trigger event contains expected values if the callback function is provided."""
        expected_result = [
            {
                "id": "1707695235850",
                "time": "2024-02-06T23:47:15.850Z",
                "message_text": "msg.",
                "message_importance": 5,
            },
            {
                "id": "1707695635401",
                "time": "2024-02-06T23:53:55.401Z",
                "message_text": "msg.",
                "message_importance": 5,
            },
        ]
        task = DataflowJobMessagesSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=lambda res: res,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
        )
        actual_result = task.execute_complete(
            context=None,
            event={
                "status": "success",
                "message": f"Detected 2 job messages for job '{TEST_JOB_ID}'",
                "result": expected_result,
            },
        )
        assert actual_result == expected_result

    def test_execute_complete_not_success_status_raises_exception(self):
        """Tests that AirflowException or AirflowSkipException is raised if the trigger event contains an error."""
        task = DataflowJobMessagesSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=None,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
        )
        with pytest.raises(AirflowException):
            task.execute_complete(
                context=None, event={"status": "error", "message": "test error message", "result": None}
            )


class TestDataflowJobAutoScalingEventsSensor:
    @pytest.mark.parametrize(
        ("job_current_state", "fail_on_terminal_state"),
        [
            (DataflowJobStatus.JOB_STATE_RUNNING, True),
            (DataflowJobStatus.JOB_STATE_RUNNING, False),
            (DataflowJobStatus.JOB_STATE_DONE, False),
        ],
    )
    @mock.patch("airflow.providers.google.cloud.sensors.dataflow.DataflowHook")
    def test_poke(self, mock_hook, job_current_state, fail_on_terminal_state):
        mock_get_job = mock_hook.return_value.get_job
        mock_fetch_job_autoscaling_events_by_id = mock_hook.return_value.fetch_job_autoscaling_events_by_id
        callback = mock.MagicMock()

        task = DataflowJobAutoScalingEventsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=callback,
            fail_on_terminal_state=fail_on_terminal_state,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_get_job.return_value = {"id": TEST_JOB_ID, "currentState": job_current_state}

        results = task.poke(mock.MagicMock())

        assert callback.return_value == results.xcom_value

        mock_hook.assert_called_once_with(
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_fetch_job_autoscaling_events_by_id.assert_called_once_with(
            job_id=TEST_JOB_ID, project_id=TEST_PROJECT_ID, location=TEST_LOCATION
        )
        callback.assert_called_once_with(mock_fetch_job_autoscaling_events_by_id.return_value)

    @mock.patch("airflow.providers.google.cloud.sensors.dataflow.DataflowHook")
    def test_poke_raise_exception_on_terminal_state(self, mock_hook):
        mock_get_job = mock_hook.return_value.get_job
        mock_fetch_job_autoscaling_events_by_id = mock_hook.return_value.fetch_job_autoscaling_events_by_id
        callback = mock.MagicMock()

        task = DataflowJobAutoScalingEventsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=callback,
            fail_on_terminal_state=True,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_get_job.return_value = {"id": TEST_JOB_ID, "currentState": DataflowJobStatus.JOB_STATE_DONE}

        with pytest.raises(
            AirflowException,
            match=f"Job with id '{TEST_JOB_ID}' is already in terminal state: "
            f"{DataflowJobStatus.JOB_STATE_DONE}",
        ):
            task.poke(mock.MagicMock())

        mock_hook.assert_called_once_with(
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
        )
        mock_fetch_job_autoscaling_events_by_id.assert_not_called()
        callback.assert_not_called()

    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook")
    def test_execute_enters_deferred_state(self, mock_hook):
        """
        Tests that AutoScalingEventTrigger will be fired when the DataflowJobAutoScalingEventSensor
        is executed and deferrable is set to True.
        """
        task = DataflowJobAutoScalingEventsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
            callback=None,
        )
        mock_hook.return_value.exists.return_value = False
        with pytest.raises(TaskDeferred) as exc:
            task.execute(None)
        assert isinstance(exc.value.trigger, DataflowJobAutoScalingEventTrigger), (
            "Trigger is not a DataflowJobAutoScalingEventTrigger"
        )

    def test_execute_complete_success_without_callback_function(self):
        """Tests that the trigger event contains expected values if no callback function is provided."""
        expected_result = []
        task = DataflowJobAutoScalingEventsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
            callback=None,
        )
        actual_message = task.execute_complete(
            context=None,
            event={
                "status": "success",
                "message": f"Detected 2 autoscaling events for job '{TEST_JOB_ID}'",
                "result": [],
            },
        )
        assert actual_message == expected_result

    def test_execute_complete_success_with_callback_function(self):
        """Tests that the trigger event contains expected values if the callback function is provided."""
        expected_result = [
            {
                "event_type": 2,
                "description": {},
                "time": "2024-02-05T13:43:31.066611771Z",
            },
            {
                "event_type": 1,
                "description": {},
                "time": "2024-02-05T13:43:31.066611771Z",
            },
        ]
        task = DataflowJobAutoScalingEventsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=lambda res: res,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
        )
        actual_result = task.execute_complete(
            context=None,
            event={
                "status": "success",
                "message": f"Detected 2 autoscaling events for job '{TEST_JOB_ID}'",
                "result": expected_result,
            },
        )
        assert actual_result == expected_result

    def test_execute_complete_not_success_status_raises_exception(self):
        """Tests that AirflowException or AirflowSkipException is raised if the trigger event contains an error."""
        task = DataflowJobAutoScalingEventsSensor(
            task_id=TEST_TASK_ID,
            job_id=TEST_JOB_ID,
            callback=None,
            fail_on_terminal_state=False,
            location=TEST_LOCATION,
            project_id=TEST_PROJECT_ID,
            gcp_conn_id=TEST_GCP_CONN_ID,
            impersonation_chain=TEST_IMPERSONATION_CHAIN,
            deferrable=True,
        )
        with pytest.raises(AirflowException):
            task.execute_complete(
                context=None,
                event={"status": "error", "message": "test error message", "result": None},
            )



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""This module contains a link for GCS Storage assets."""

from __future__ import annotations

from airflow.providers.google.cloud.links.base import BaseGoogleLink

BASE_LINK = "https://console.cloud.google.com"
GCS_STORAGE_LINK = BASE_LINK + "/storage/browser/{uri};tab=objects?project={project_id}"
GCS_FILE_DETAILS_LINK = BASE_LINK + "/storage/browser/_details/{uri};tab=live_object?project={project_id}"


class StorageLink(BaseGoogleLink):
    """Helper class for constructing GCS Storage link."""

    name = "GCS Storage"
    key = "storage_conf"
    format_str = GCS_STORAGE_LINK


class FileDetailsLink(BaseGoogleLink):
    """Helper class for constructing GCS file details link."""

    name = "GCS File Details"
    key = "file_details"
    format_str = GCS_FILE_DETAILS_LINK



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""
Example Airflow DAG that starts, stops and sets the machine type of a Google Compute
Engine instance.

"""

from __future__ import annotations

import os
from datetime import datetime

from airflow.models.dag import DAG
from airflow.providers.google.cloud.hooks.compute_ssh import ComputeEngineSSHHook
from airflow.providers.google.cloud.operators.compute import (
    ComputeEngineDeleteInstanceOperator,
    ComputeEngineInsertInstanceOperator,
)
from airflow.providers.ssh.operators.ssh import SSHOperator

try:
    from airflow.sdk import TriggerRule
except ImportError:
    # Compatibility for Airflow < 3.1
    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]

from system.google import DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID

# [START howto_operator_gce_args_common]
ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT") or DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID

DAG_ID = "cloud_compute_ssh_os_login"
LOCATION = "europe-west2-b"
REGION = "europe-west2"
GCE_INSTANCE_NAME = "instance-ssh-test-oslogin"
SHORT_MACHINE_TYPE_NAME = "n1-standard-1"
GCE_INSTANCE_BODY = {
    "name": GCE_INSTANCE_NAME,
    "machine_type": f"zones/{LOCATION}/machineTypes/{SHORT_MACHINE_TYPE_NAME}",
    "metadata": {
        "items": [
            {
                "key": "enable-oslogin",
                "value": "TRUE",
            }
        ]
    },
    "disks": [
        {
            "boot": True,
            "auto_delete": True,
            "device_name": GCE_INSTANCE_NAME,
            "initialize_params": {
                "disk_size_gb": "10",
                "disk_type": f"zones/{LOCATION}/diskTypes/pd-balanced",
                "source_image": "projects/debian-cloud/global/images/debian-12-bookworm-v20240611",
            },
        }
    ],
    "network_interfaces": [
        {
            "access_configs": [{"name": "External NAT", "network_tier": "PREMIUM"}],
            "stack_type": "IPV4_ONLY",
            "subnetwork": f"regions/{REGION}/subnetworks/default",
        }
    ],
}
# [END howto_operator_gce_args_common]

with DAG(
    DAG_ID,
    schedule="@once",
    start_date=datetime(2021, 1, 1),
    catchup=False,
    tags=["example", "compute-ssh", "os-login"],
) as dag:
    # [START howto_operator_gce_insert]
    gce_instance_insert = ComputeEngineInsertInstanceOperator(
        task_id="gcp_compute_create_instance_task",
        project_id=PROJECT_ID,
        zone=LOCATION,
        body=GCE_INSTANCE_BODY,
    )
    # [END howto_operator_gce_insert]

    # [START howto_execute_command_on_remote_1]
    os_login_task1 = SSHOperator(
        task_id="os_login_task1",
        ssh_hook=ComputeEngineSSHHook(
            user="username",
            instance_name=GCE_INSTANCE_NAME,
            zone=LOCATION,
            project_id=PROJECT_ID,
            use_oslogin=True,
            use_iap_tunnel=False,
            cmd_timeout=1,
        ),
        command="echo os_login1",
    )
    # [END howto_execute_command_on_remote_1]

    # [START howto_execute_command_on_remote_2]
    os_login_task2 = SSHOperator(
        task_id="os_login_task2",
        ssh_hook=ComputeEngineSSHHook(
            user="username",
            instance_name=GCE_INSTANCE_NAME,
            zone=LOCATION,
            use_oslogin=True,
            use_iap_tunnel=False,
            cmd_timeout=1,
        ),
        command="echo os_login2",
    )
    # [END howto_execute_command_on_remote_2]

    # [START howto_operator_gce_delete_no_project_id]
    gce_instance_delete = ComputeEngineDeleteInstanceOperator(
        task_id="gcp_compute_delete_instance_task",
        zone=LOCATION,
        resource_id=GCE_INSTANCE_NAME,
        trigger_rule=TriggerRule.ALL_DONE,
    )
    # [END howto_operator_gce_delete_no_project_id]

    (
        # TEST SETUP
        gce_instance_insert
        # TEST BODY
        >> os_login_task1
        >> os_login_task2
        # TEST TEARDOWN
        >> gce_instance_delete
    )

    # ### Everything below this line is not part of example ###
    # ### Just for system tests purpose ###
    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()


from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from airflow.providers.google.cloud.transfers.http_to_gcs import HttpToGCSOperator

TASK_ID = "test-http-to-gcs-operator"
GCP_CONN_ID = "GCP_CONN_ID"
HTTP_CONN_ID = "HTTP_CONN_ID"
IMPERSONATION_CHAIN = ["ACCOUNT_1", "ACCOUNT_2", "ACCOUNT_3"]

TEST_BUCKET = "test-bucket"
DESTINATION_PATH_FILE = "destination_dir/copy.txt"
ENDPOINT = "/"
HEADERS = {"header_key": "header_value"}
DATA = {"some": "data"}
EXTRA_OPTIONS = {"check_response": False}
DEFAULT_HTTP_METHOD = "GET"
NUM_MAX_ATTEMPTS = 3
TCP_KEEP_ALIVE_IDLE = 120
TCP_KEEP_ALIVE_COUNT = 20
TCP_KEEP_ALIVE_INTERVAL = 30


class TestHttpToGCSOperator:
    def test_init(self):
        operator = HttpToGCSOperator(
            task_id="http_to_gcs_operator",
            http_conn_id=HTTP_CONN_ID,
            endpoint=ENDPOINT,
            object_name=DESTINATION_PATH_FILE,
            bucket_name=TEST_BUCKET,
        )
        assert operator.endpoint == ENDPOINT
        assert operator.object_name == DESTINATION_PATH_FILE
        assert operator.bucket_name == TEST_BUCKET
        assert operator.http_conn_id == HTTP_CONN_ID

    @mock.patch("airflow.providers.google.cloud.transfers.http_to_gcs.GCSHook")
    @mock.patch("airflow.providers.google.cloud.transfers.http_to_gcs.HttpHook")
    def test_execute_copy_single_file(self, http_hook, gcs_hook):
        task = HttpToGCSOperator(
            task_id="http_to_gcs_operator",
            http_conn_id=HTTP_CONN_ID,
            endpoint=ENDPOINT,
            headers=HEADERS,
            data=DATA,
            extra_options=EXTRA_OPTIONS,
            object_name=DESTINATION_PATH_FILE,
            bucket_name=TEST_BUCKET,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        task.execute(None)

        # GCS
        gcs_hook.assert_called_once_with(gcp_conn_id=GCP_CONN_ID, impersonation_chain=IMPERSONATION_CHAIN)
        task.gcs_hook.upload.assert_called_once_with(
            bucket_name=TEST_BUCKET,
            object_name=DESTINATION_PATH_FILE,
            data=task.http_hook.run.return_value.content,
            mime_type=None,
            gzip=False,
            encoding=task.http_hook.run.return_value.encoding,
            chunk_size=None,
            timeout=None,
            num_max_attempts=NUM_MAX_ATTEMPTS,
            metadata=None,
            cache_control=None,
            user_project=None,
        )

        # HTTP
        http_hook.assert_called_once_with(
            DEFAULT_HTTP_METHOD,
            http_conn_id=HTTP_CONN_ID,
            auth_type=None,
            tcp_keep_alive=True,
            tcp_keep_alive_idle=TCP_KEEP_ALIVE_IDLE,
            tcp_keep_alive_count=TCP_KEEP_ALIVE_COUNT,
            tcp_keep_alive_interval=TCP_KEEP_ALIVE_INTERVAL,
        )
        task.http_hook.run.assert_called_once_with(
            endpoint=ENDPOINT, headers=HEADERS, data=DATA, extra_options=EXTRA_OPTIONS
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""
Example Airflow DAG for testing Google Dataflow to create pipelines.
"""

from __future__ import annotations

import os
from datetime import datetime, timedelta

from airflow.models.dag import DAG
from airflow.providers.google.cloud.operators.dataflow import (
    DataflowCreatePipelineOperator,
    DataflowDeletePipelineOperator,
    DataflowStopJobOperator,
)
from airflow.providers.google.cloud.operators.pubsub import (
    PubSubCreateSubscriptionOperator,
    PubSubCreateTopicOperator,
    PubSubDeleteSubscriptionOperator,
    PubSubDeleteTopicOperator,
)

try:
    from airflow.sdk import TriggerRule
except ImportError:
    # Compatibility for Airflow < 3.1
    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]

DAG_ID = "dataflow_pipeline_streaming"
ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
GCP_PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT", "default")
GCP_LOCATION = "europe-central2"

PIPELINE_NAME = f"{DAG_ID}-{ENV_ID}".replace("_", "-")
PIPELINE_JOB_NAME = f"{DAG_ID}-job".replace("_", "-")

PIPELINE_TYPE = "PIPELINE_TYPE_STREAMING"

RESOURCE_DATA_BUCKET = "airflow-system-tests-resources"
BUCKET_NAME = f"bucket_{DAG_ID}_{ENV_ID}".replace("-", "_")

INPUT_TOPIC_ID = "taxirides-realtime"
INPUT_TOPIC_PROJECT_ID = "pubsub-public-data"
INPUT_TOPIC = f"projects/{INPUT_TOPIC_PROJECT_ID}/topics/{INPUT_TOPIC_ID}"
SUBSCRIPTION_NAME = "taxirides-sub"

OUTPUT_TOPIC_ID = f"df-tp-out-{ENV_ID}"
OUTPUT_TOPIC = f"projects/{GCP_PROJECT_ID}/topics/{OUTPUT_TOPIC_ID}"


with DAG(
    DAG_ID,
    schedule="@once",
    start_date=datetime(2025, 1, 1),
    catchup=False,
    tags=["example", "dataflow", "pipelines", "streaming"],
) as dag:
    create_output_pub_sub_topic = PubSubCreateTopicOperator(
        task_id="create_topic", topic=OUTPUT_TOPIC_ID, project_id=GCP_PROJECT_ID, fail_if_exists=False
    )
    create_subscription = PubSubCreateSubscriptionOperator(
        task_id="create_subscription",
        project_id=INPUT_TOPIC_PROJECT_ID,
        subscription_project_id=GCP_PROJECT_ID,
        topic=INPUT_TOPIC_ID,
        subscription=SUBSCRIPTION_NAME,
    )

    # [START howto_operator_create_dataflow_pipeline_streaming]
    create_pipeline = DataflowCreatePipelineOperator(
        task_id="create_pipeline",
        project_id=GCP_PROJECT_ID,
        location=GCP_LOCATION,
        body={
            "name": f"projects/{GCP_PROJECT_ID}/locations/{GCP_LOCATION}/pipelines/{PIPELINE_NAME}",
            "type": PIPELINE_TYPE,
            "workload": {
                "dataflowLaunchTemplateRequest": {
                    "projectId": GCP_PROJECT_ID,
                    "location": GCP_LOCATION,
                    "gcsPath": "gs://dataflow-templates-europe-central2/latest/Cloud_PubSub_to_Cloud_PubSub",
                    "launchParameters": {
                        "jobName": PIPELINE_JOB_NAME,
                        "parameters": {
                            "inputSubscription": f"projects/{GCP_PROJECT_ID}/subscriptions/{SUBSCRIPTION_NAME}",
                            "outputTopic": OUTPUT_TOPIC,
                        },
                    },
                }
            },
        },
    )
    # [END howto_operator_create_dataflow_pipeline_streaming]

    delete_pipeline = DataflowDeletePipelineOperator(
        task_id="delete_pipeline",
        pipeline_name=PIPELINE_NAME,
        project_id=GCP_PROJECT_ID,
        location=GCP_LOCATION,
        trigger_rule=TriggerRule.ALL_DONE,
    )

    delete_output_pub_sub_topic = PubSubDeleteTopicOperator(
        task_id="delete_out_topic",
        topic=OUTPUT_TOPIC_ID,
        project_id=GCP_PROJECT_ID,
        trigger_rule=TriggerRule.ALL_DONE,
    )

    delete_subscription = PubSubDeleteSubscriptionOperator(
        task_id="delete_subscription",
        project_id=GCP_PROJECT_ID,
        subscription=SUBSCRIPTION_NAME,
        trigger_rule=TriggerRule.ALL_DONE,
    )

    stop_dataflow_job = DataflowStopJobOperator(
        task_id="stop_dataflow_job",
        location=GCP_LOCATION,
        job_name_prefix=PIPELINE_NAME[:20],
        drain_pipeline=False,
        retries=3,
        retry_delay=timedelta(seconds=45),
    )

    (
        # TEST SETUP
        [create_output_pub_sub_topic, create_subscription]
        # TEST BODY
        >> create_pipeline
        # TEST TEARDOWN
        >> [delete_output_pub_sub_topic, delete_subscription]
        >> delete_pipeline
        >> stop_dataflow_job
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "teardown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()


from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example Airflow DAG that shows how to use CampaignManager.

This system test relies on a service account with proper settings in Campaign Manager 360.
That's why before running this system test locally, make sure your service account corresponds all the
secrets that the DAG reads. If your service account doesn't have access but you know another one which has
then simply specify it in the environment variable CM360_IMPERSONATION_CHAIN.
"""

from __future__ import annotations

import json
import logging
import os
import time
import uuid
from datetime import datetime
from typing import Any, cast

try:
    from airflow.sdk import task
except ImportError:
    # Airflow 2 path
    from airflow.decorators import task  # type: ignore[attr-defined,no-redef]
from airflow.models.dag import DAG
from airflow.models.xcom_arg import XComArg
from airflow.providers.google.cloud.operators.gcs import GCSCreateBucketOperator, GCSDeleteBucketOperator
from airflow.providers.google.common.utils.get_secret import get_secret
from airflow.providers.google.marketing_platform.operators.campaign_manager import (
    GoogleCampaignManagerBatchInsertConversionsOperator,
    GoogleCampaignManagerBatchUpdateConversionsOperator,
    GoogleCampaignManagerDeleteReportOperator,
    GoogleCampaignManagerDownloadReportOperator,
    GoogleCampaignManagerInsertReportOperator,
    GoogleCampaignManagerRunReportOperator,
)
from airflow.providers.google.marketing_platform.sensors.campaign_manager import (
    GoogleCampaignManagerReportSensor,
)

try:
    from airflow.sdk import TriggerRule
except ImportError:
    # Compatibility for Airflow < 3.1
    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]

from system.google import DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID
from system.google.gcp_api_client_helpers import create_airflow_connection, delete_airflow_connection

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT") or DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID
CM360_IMPERSONATION_CHAIN = os.environ.get("IMPERSONATION_CHAIN", None)

DAG_ID = "campaign_manager"

IS_COMPOSER = bool(os.environ.get("COMPOSER_ENVIRONMENT", ""))

SECRET_ACCOUNT_ID = "cm360_account_id"
SECRET_DCLID = "cm360_dclid"
SECRET_ENCRYPTION_ENTITY_ID = "cm360_encryption_entity_id"
SECRET_FLOODLIGHT_ACTIVITY_ID = "cm360_floodlight_activity_id"
SECRET_FLOODLIGHT_CONFIGURATION_ID = "cm360_floodlight_configuration_id"
SECRET_USER_PROFILE_ID = "cm360_user_profile_id"

ACCOUNT_ID = "{{ task_instance.xcom_pull('get_account_id') }}"
DCLID = "{{ task_instance.xcom_pull('get_dclid') }}"
ENCRYPTION_ENTITY_ID = "{{ task_instance.xcom_pull('get_encryption_entity_id') }}"
FLOODLIGHT_ACTIVITY_ID = "{{ task_instance.xcom_pull('get_floodlight_activity_id') }}"
FLOODLIGHT_CONFIGURATION_ID = "{{ task_instance.xcom_pull('get_floodlight_configuration_id') }}"
USER_PROFILE_ID = "{{ task_instance.xcom_pull('get_user_profile_id') }}"

BUCKET_NAME = f"bucket_{DAG_ID}_{ENV_ID}"
REPORT_NAME = f"report_{DAG_ID}_{ENV_ID}"
FILE_NAME = f"file_{DAG_ID}_{ENV_ID}"
FORMAT = "CSV"
CONNECTION_ID = f"connection_{DAG_ID}_{ENV_ID}"


# For more information, please check
# https://developers.google.com/doubleclick-advertisers/rest/v4/reports#type
REPORT = {
    "kind": "dfareporting#report",
    "type": "STANDARD",
    "name": REPORT_NAME,
    "fileName": FILE_NAME,
    "accountId": ACCOUNT_ID,
    "format": FORMAT,
    "criteria": {
        "dateRange": {
            "kind": "dfareporting#dateRange",
            "relativeDateRange": "LAST_365_DAYS",
        },
        "dimensions": [{"kind": "dfareporting#sortedDimension", "name": "campaign"}],
        "metricNames": ["activeViewImpressionDistributionViewable"],
    },
}

# For more information, please check
# https://developers.google.com/doubleclick-advertisers/rest/v4/Conversion
CONVERSION = {
    "kind": "dfareporting#conversion",
    "floodlightActivityId": FLOODLIGHT_ACTIVITY_ID,
    "floodlightConfigurationId": FLOODLIGHT_CONFIGURATION_ID,
    "dclid": DCLID,
    "ordinal": "0",
    "quantity": 42,
    "value": 123.4,
    "timestampMicros": int(time.time()) * 1000000,
    "customVariables": [
        {
            "kind": "dfareporting#customFloodlightVariable",
            "type": "U4",
            "value": "value",
        }
    ],
}

CONVERSION_UPDATE = {
    "kind": "dfareporting#conversion",
    "floodlightActivityId": FLOODLIGHT_ACTIVITY_ID,
    "floodlightConfigurationId": FLOODLIGHT_CONFIGURATION_ID,
    "dclid": DCLID,
    "ordinal": "0",
    "quantity": 42,
    "value": 123.4,
}


log = logging.getLogger(__name__)


with DAG(
    DAG_ID,
    schedule="@once",  # Override to match your needs,
    start_date=datetime(2021, 1, 1),
    catchup=False,
    tags=["example", "campaign"],
) as dag:

    @task
    def create_connection(connection_id: str) -> None:
        extras = {
            "scope": "https://www.googleapis.com/auth/cloud-platform,"
            "https://www.googleapis.com/auth/ddmconversions,"
            "https://www.googleapis.com/auth/dfareporting",
        }
        if CM360_IMPERSONATION_CHAIN:
            extras["impersonation_chain"] = CM360_IMPERSONATION_CHAIN

        conn_extra_json = json.dumps(extras)
        connection: dict[str, Any] = {"conn_type": "google_cloud_platform", "extra": conn_extra_json}
        create_airflow_connection(
            connection_id=connection_id,
            connection_conf=connection,
            is_composer=IS_COMPOSER,
        )

    @task
    def get_account_id():
        return get_secret(secret_id=SECRET_ACCOUNT_ID)

    @task
    def get_dclid():
        return get_secret(secret_id=SECRET_DCLID)

    @task
    def get_encryption_entity_id():
        return get_secret(secret_id=SECRET_ENCRYPTION_ENTITY_ID)

    @task
    def get_floodlight_activity_id():
        return get_secret(secret_id=SECRET_FLOODLIGHT_ACTIVITY_ID)

    @task
    def get_floodlight_configuration_id():
        return get_secret(secret_id=SECRET_FLOODLIGHT_CONFIGURATION_ID)

    @task
    def get_user_profile_id():
        return get_secret(secret_id=SECRET_USER_PROFILE_ID)

    create_bucket = GCSCreateBucketOperator(
        task_id="create_bucket", bucket_name=BUCKET_NAME, project_id=PROJECT_ID
    )

    # [START howto_campaign_manager_insert_report_operator]
    create_report = GoogleCampaignManagerInsertReportOperator(
        profile_id=USER_PROFILE_ID,
        report=REPORT,
        task_id="create_report",
        gcp_conn_id=CONNECTION_ID,
    )
    report_id = cast("str", XComArg(create_report, key="report_id"))
    # [END howto_campaign_manager_insert_report_operator]

    # [START howto_campaign_manager_run_report_operator]
    run_report = GoogleCampaignManagerRunReportOperator(
        profile_id=USER_PROFILE_ID,
        report_id=report_id,
        task_id="run_report",
        gcp_conn_id=CONNECTION_ID,
    )
    file_id = cast("str", XComArg(run_report, key="file_id"))
    # [END howto_campaign_manager_run_report_operator]

    # [START howto_campaign_manager_wait_for_operation]
    wait_for_report = GoogleCampaignManagerReportSensor(
        task_id="wait_for_report",
        profile_id=USER_PROFILE_ID,
        report_id=report_id,
        file_id=file_id,
        gcp_conn_id=CONNECTION_ID,
    )
    # [END howto_campaign_manager_wait_for_operation]

    # [START howto_campaign_manager_get_report_operator]
    report_name = f"reports/report_{str(uuid.uuid1())}"
    get_report = GoogleCampaignManagerDownloadReportOperator(
        task_id="get_report",
        profile_id=USER_PROFILE_ID,
        report_id=report_id,
        file_id=file_id,
        report_name=report_name,
        bucket_name=BUCKET_NAME,
        gcp_conn_id=CONNECTION_ID,
    )
    # [END howto_campaign_manager_get_report_operator]

    # [START howto_campaign_manager_delete_report_operator]
    delete_report = GoogleCampaignManagerDeleteReportOperator(
        profile_id=USER_PROFILE_ID,
        report_name=REPORT_NAME,
        task_id="delete_report",
        trigger_rule=TriggerRule.ALL_DONE,
        gcp_conn_id=CONNECTION_ID,
    )
    # [END howto_campaign_manager_delete_report_operator]

    delete_bucket = GCSDeleteBucketOperator(
        task_id="delete_bucket", bucket_name=BUCKET_NAME, trigger_rule=TriggerRule.ALL_DONE
    )

    # [START howto_campaign_manager_insert_conversions]
    insert_conversion = GoogleCampaignManagerBatchInsertConversionsOperator(
        task_id="insert_conversion",
        profile_id=USER_PROFILE_ID,
        conversions=[CONVERSION],
        encryption_source="AD_SERVING",
        encryption_entity_type="DCM_ADVERTISER",
        encryption_entity_id=ENCRYPTION_ENTITY_ID,  # type: ignore[arg-type]
        gcp_conn_id=CONNECTION_ID,
    )
    # [END howto_campaign_manager_insert_conversions]

    # [START howto_campaign_manager_update_conversions]
    update_conversion = GoogleCampaignManagerBatchUpdateConversionsOperator(
        task_id="update_conversion",
        profile_id=USER_PROFILE_ID,
        conversions=[CONVERSION_UPDATE],
        encryption_source="AD_SERVING",
        encryption_entity_type="DCM_ADVERTISER",
        encryption_entity_id=ENCRYPTION_ENTITY_ID,  # type: ignore[arg-type]
        max_failed_updates=1,
        gcp_conn_id=CONNECTION_ID,
    )
    # [END howto_campaign_manager_update_conversions]

    @task(task_id="delete_connection")
    def delete_connection(connection_id: str) -> None:
        delete_airflow_connection(connection_id=connection_id, is_composer=IS_COMPOSER)

    (
        # TEST SETUP
        create_connection(connection_id=CONNECTION_ID)
        >> [
            get_account_id(),
            get_dclid(),
            get_encryption_entity_id(),
            get_floodlight_activity_id(),
            get_floodlight_configuration_id(),
            get_user_profile_id(),
        ]
        >> create_bucket
        # TEST BODY
        >> create_report
        >> run_report
        >> wait_for_report
        >> get_report
        >> insert_conversion
        >> update_conversion
        # TEST TEARDOWN
        >> delete_report
        >> delete_bucket
        >> delete_connection(connection_id=CONNECTION_ID)
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG

    # Excluding sensor because we expect it to fail due to cancelled operation
    [
        task
        for task in dag.tasks
        if task.task_id not in ["insert_conversion", "update_conversion", "delete_connection"]
    ] >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from google.api_core.gapic_v1.method import DEFAULT

from airflow.providers.google.cloud.hooks.managed_kafka import ManagedKafkaHook

from unit.google.cloud.utils.base_gcp_mock import (
    mock_base_gcp_hook_default_project_id,
    mock_base_gcp_hook_no_default_project_id,
)

TEST_GCP_CONN_ID: str = "test-gcp-conn-id"
TEST_LOCATION: str = "test-location"
TEST_PROJECT_ID: str = "test-project-id"
TEST_CLUSTER_ID: str = "test-cluster-id"
TEST_CLUSTER: dict = {
    "gcp_config": {
        "access_config": {
            "network_configs": {
                "subnet": "subnet_value",
            },
        },
    },
    "capacity_config": {
        "vcpu_count": 1094,
        "memory_bytes": 1311,
    },
}
TEST_CLUSTER_UPDATE_MASK: dict = {"paths": ["gcp_config.access_config.network_configs.subnet"]}
TEST_UPDATED_CLUSTER: dict = {
    "gcp_config": {
        "access_config": {
            "network_configs": {
                "subnet": "new_subnet_value",
            },
        },
    },
}

TEST_TOPIC_ID: str = "test-topic-id"
TEST_TOPIC: dict = {
    "partition_count": 1634,
    "replication_factor": 1912,
}
TEST_TOPIC_UPDATE_MASK: dict = {"paths": ["partition_count"]}
TEST_UPDATED_TOPIC: dict = {
    "partition_count": 2000,
    "replication_factor": 1912,
}

TEST_CONSUMER_GROUP_ID: str = "test-consumer-group-id"

BASE_STRING = "airflow.providers.google.common.hooks.base_google.{}"
MANAGED_KAFKA_STRING = "airflow.providers.google.cloud.hooks.managed_kafka.{}"


class TestManagedKafkaWithDefaultProjectIdHook:
    def setup_method(self):
        with mock.patch(
            BASE_STRING.format("GoogleBaseHook.__init__"), new=mock_base_gcp_hook_default_project_id
        ):
            self.hook = ManagedKafkaHook(gcp_conn_id=TEST_GCP_CONN_ID)

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_create_cluster(self, mock_client) -> None:
        self.hook.create_cluster(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster=TEST_CLUSTER,
            cluster_id=TEST_CLUSTER_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.create_cluster.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                cluster=TEST_CLUSTER,
                cluster_id=TEST_CLUSTER_ID,
                request_id=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(TEST_PROJECT_ID, TEST_LOCATION)

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_delete_cluster(self, mock_client) -> None:
        self.hook.delete_cluster(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.delete_cluster.assert_called_once_with(
            request=dict(name=mock_client.return_value.cluster_path.return_value, request_id=None),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_get_cluster(self, mock_client) -> None:
        self.hook.get_cluster(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.get_cluster.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.cluster_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_update_cluster(self, mock_client) -> None:
        self.hook.update_cluster(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster=TEST_UPDATED_CLUSTER,
            cluster_id=TEST_CLUSTER_ID,
            update_mask=TEST_CLUSTER_UPDATE_MASK,
        )
        mock_client.assert_called_once()
        mock_client.return_value.update_cluster.assert_called_once_with(
            request=dict(
                update_mask=TEST_CLUSTER_UPDATE_MASK,
                cluster={
                    "name": mock_client.return_value.cluster_path.return_value,
                    **TEST_UPDATED_CLUSTER,
                },
                request_id=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_list_clusters(self, mock_client) -> None:
        self.hook.list_clusters(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
        )
        mock_client.assert_called_once()
        mock_client.return_value.list_clusters.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                page_size=None,
                page_token=None,
                filter=None,
                order_by=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(TEST_PROJECT_ID, TEST_LOCATION)

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_create_topic(self, mock_client) -> None:
        self.hook.create_topic(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            topic_id=TEST_TOPIC_ID,
            topic=TEST_TOPIC,
        )
        mock_client.assert_called_once()
        mock_client.return_value.create_topic.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.cluster_path.return_value,
                topic_id=TEST_TOPIC_ID,
                topic=TEST_TOPIC,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_delete_topic(self, mock_client) -> None:
        self.hook.delete_topic(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            topic_id=TEST_TOPIC_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.delete_topic.assert_called_once_with(
            request=dict(name=mock_client.return_value.topic_path.return_value),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.topic_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID, TEST_TOPIC_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_get_topic(self, mock_client) -> None:
        self.hook.get_topic(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            topic_id=TEST_TOPIC_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.get_topic.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.topic_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.topic_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_LOCATION,
            TEST_CLUSTER_ID,
            TEST_TOPIC_ID,
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_update_topic(self, mock_client) -> None:
        self.hook.update_topic(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            topic_id=TEST_TOPIC_ID,
            topic=TEST_UPDATED_TOPIC,
            update_mask=TEST_TOPIC_UPDATE_MASK,
        )
        mock_client.assert_called_once()
        mock_client.return_value.update_topic.assert_called_once_with(
            request=dict(
                update_mask=TEST_TOPIC_UPDATE_MASK,
                topic={
                    "name": mock_client.return_value.topic_path.return_value,
                    **TEST_UPDATED_TOPIC,
                },
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.topic_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID, TEST_TOPIC_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_list_topics(self, mock_client) -> None:
        self.hook.list_topics(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.list_topics.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.cluster_path.return_value,
                page_size=None,
                page_token=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_delete_consumer_group(self, mock_client) -> None:
        self.hook.delete_consumer_group(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            consumer_group_id=TEST_CONSUMER_GROUP_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.delete_consumer_group.assert_called_once_with(
            request=dict(name=mock_client.return_value.consumer_group_path.return_value),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.consumer_group_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID, TEST_CONSUMER_GROUP_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_get_consumer_group(self, mock_client) -> None:
        self.hook.get_consumer_group(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            consumer_group_id=TEST_CONSUMER_GROUP_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.get_consumer_group.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.consumer_group_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.consumer_group_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_LOCATION,
            TEST_CLUSTER_ID,
            TEST_CONSUMER_GROUP_ID,
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_update_consumer_group(self, mock_client) -> None:
        self.hook.update_consumer_group(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            consumer_group_id=TEST_CONSUMER_GROUP_ID,
            consumer_group={},
            update_mask={},
        )
        mock_client.assert_called_once()
        mock_client.return_value.update_consumer_group.assert_called_once_with(
            request=dict(
                update_mask={},
                consumer_group={
                    "name": mock_client.return_value.consumer_group_path.return_value,
                    **{},
                },
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.consumer_group_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID, TEST_CONSUMER_GROUP_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_list_consumer_groups(self, mock_client) -> None:
        self.hook.list_consumer_groups(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.list_consumer_groups.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.cluster_path.return_value,
                page_size=None,
                page_token=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )


class TestManagedKafkaWithoutDefaultProjectIdHook:
    def setup_method(self):
        with mock.patch(
            BASE_STRING.format("GoogleBaseHook.__init__"), new=mock_base_gcp_hook_no_default_project_id
        ):
            self.hook = ManagedKafkaHook(gcp_conn_id=TEST_GCP_CONN_ID)

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_create_cluster(self, mock_client) -> None:
        self.hook.create_cluster(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster=TEST_CLUSTER,
            cluster_id=TEST_CLUSTER_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.create_cluster.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                cluster=TEST_CLUSTER,
                cluster_id=TEST_CLUSTER_ID,
                request_id=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(TEST_PROJECT_ID, TEST_LOCATION)

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_delete_cluster(self, mock_client) -> None:
        self.hook.delete_cluster(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.delete_cluster.assert_called_once_with(
            request=dict(name=mock_client.return_value.cluster_path.return_value, request_id=None),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_get_cluster(self, mock_client) -> None:
        self.hook.get_cluster(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.get_cluster.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.cluster_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_update_cluster(self, mock_client) -> None:
        self.hook.update_cluster(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster=TEST_UPDATED_CLUSTER,
            cluster_id=TEST_CLUSTER_ID,
            update_mask=TEST_CLUSTER_UPDATE_MASK,
        )
        mock_client.assert_called_once()
        mock_client.return_value.update_cluster.assert_called_once_with(
            request=dict(
                update_mask=TEST_CLUSTER_UPDATE_MASK,
                cluster={
                    "name": mock_client.return_value.cluster_path.return_value,
                    **TEST_UPDATED_CLUSTER,
                },
                request_id=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_list_clusters(self, mock_client) -> None:
        self.hook.list_clusters(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
        )
        mock_client.assert_called_once()
        mock_client.return_value.list_clusters.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                page_size=None,
                page_token=None,
                filter=None,
                order_by=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(TEST_PROJECT_ID, TEST_LOCATION)

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_create_topic(self, mock_client) -> None:
        self.hook.create_topic(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            topic_id=TEST_TOPIC_ID,
            topic=TEST_TOPIC,
        )
        mock_client.assert_called_once()
        mock_client.return_value.create_topic.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.cluster_path.return_value,
                topic_id=TEST_TOPIC_ID,
                topic=TEST_TOPIC,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_delete_topic(self, mock_client) -> None:
        self.hook.delete_topic(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            topic_id=TEST_TOPIC_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.delete_topic.assert_called_once_with(
            request=dict(name=mock_client.return_value.topic_path.return_value),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.topic_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_LOCATION,
            TEST_CLUSTER_ID,
            TEST_TOPIC_ID,
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_get_topic(self, mock_client) -> None:
        self.hook.get_topic(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            topic_id=TEST_TOPIC_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.get_topic.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.topic_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.topic_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_LOCATION,
            TEST_CLUSTER_ID,
            TEST_TOPIC_ID,
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_update_topic(self, mock_client) -> None:
        self.hook.update_topic(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            topic_id=TEST_TOPIC_ID,
            topic=TEST_UPDATED_TOPIC,
            update_mask=TEST_TOPIC_UPDATE_MASK,
        )
        mock_client.assert_called_once()
        mock_client.return_value.update_topic.assert_called_once_with(
            request=dict(
                update_mask=TEST_TOPIC_UPDATE_MASK,
                topic={
                    "name": mock_client.return_value.topic_path.return_value,
                    **TEST_UPDATED_TOPIC,
                },
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.topic_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID, TEST_TOPIC_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_list_topics(self, mock_client) -> None:
        self.hook.list_topics(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.list_topics.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.cluster_path.return_value,
                page_size=None,
                page_token=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_delete_consumer_group(self, mock_client) -> None:
        self.hook.delete_consumer_group(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            consumer_group_id=TEST_CONSUMER_GROUP_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.delete_consumer_group.assert_called_once_with(
            request=dict(name=mock_client.return_value.consumer_group_path.return_value),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.consumer_group_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_LOCATION,
            TEST_CLUSTER_ID,
            TEST_CONSUMER_GROUP_ID,
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_get_consumer_group(self, mock_client) -> None:
        self.hook.get_consumer_group(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            consumer_group_id=TEST_CONSUMER_GROUP_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.get_consumer_group.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.consumer_group_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.consumer_group_path.assert_called_once_with(
            TEST_PROJECT_ID,
            TEST_LOCATION,
            TEST_CLUSTER_ID,
            TEST_CONSUMER_GROUP_ID,
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_update_consumer_group(self, mock_client) -> None:
        self.hook.update_consumer_group(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
            consumer_group_id=TEST_CONSUMER_GROUP_ID,
            consumer_group={},
            update_mask={},
        )
        mock_client.assert_called_once()
        mock_client.return_value.update_consumer_group.assert_called_once_with(
            request=dict(
                update_mask={},
                consumer_group={
                    "name": mock_client.return_value.consumer_group_path.return_value,
                    **{},
                },
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.consumer_group_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID, TEST_CONSUMER_GROUP_ID
        )

    @mock.patch(MANAGED_KAFKA_STRING.format("ManagedKafkaHook.get_managed_kafka_client"))
    def test_list_consumer_groups(self, mock_client) -> None:
        self.hook.list_consumer_groups(
            project_id=TEST_PROJECT_ID,
            location=TEST_LOCATION,
            cluster_id=TEST_CLUSTER_ID,
        )
        mock_client.assert_called_once()
        mock_client.return_value.list_consumer_groups.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.cluster_path.return_value,
                page_size=None,
                page_token=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.cluster_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_LOCATION, TEST_CLUSTER_ID
        )
